#!perl -w
use strict;

# $Id$
#
# Copyright (c) 1997 Roderick Schertler.  All rights reserved.  This
# program is free software; you can redistribute it and/or modify it
# under the same terms as Perl itself.

BEGIN {
    $| = 1;
    #print "1..1\n";
}

use User::utent qw(:type :msg :path);

sub ok {
    my ($n, $result, @info) = @_;
    if ($result) {
    	print "ok $n\n";
    }
    else {
    	print "not ok $n\n";
	print "# ", @info, "\n" if @info;
    }
}

sub show {
    my $ut = shift;
    printf "%-8s %-4s %-12s %5d %3d %-16s ",
    	$ut->ut_user, $ut->ut_id, $ut->ut_line,
    	$ut->ut_pid, $ut->ut_type, $ut->ut_host;
    if ($ut->ut_type == DEAD_PROCESS) {
	printf "%10d %d/%d", $ut->ut_time, $ut->e_termination, $ut->e_exit;
    }
    else {
	print scalar localtime $ut->ut_time;
    }
    print "\n";
}

BEGIN {
    for my $sym (qw(setutent endutent utmpname
		    getutent getutid getutline
		    pututline clear_last_ut)) {
	no strict 'refs';
	*$sym = \&{ "User::utent::utmp::$sym" };
    }
}

die "no func specified" unless @ARGV;
my $func = shift;

if ($func == 1) {
    my $x = new User::utent::utmp;
    print ":", $x->ut_host("host set"), ":\n";;
    print ":", $x->ut_host("second host set"), ":\n";;
    print ":", $x->ut_host, ":\n";;
    print ":", $x->ut_pid(1), ":\n";;
    print ":", $x->ut_pid(23), ":\n";;
    print ":", $x->ut_pid, ":\n";;
}

elsif ($func == 2) {
    while (my $x = getutent) {
	show $x;
    }
}

elsif ($func == 3) {
    while (my $x = getutid 7, "rl10") {
	show $x;
    }
    print "rewinding\n";
    setutent;
    while (my $x = getutid 7, "rl10") {
	show $x;
    }
}

elsif ($func == 4) {
    utmpname '/var/adm/wtmp' or die;
    while (my $x = getutent) {
	show $x;
    }
}

elsif ($func == 5) {
    utmpname '/var/adm/wtmp' or die;
    while (my $x = getutline 'pts/12') {
	show $x;
	clear_last_ut;
    }
}

# This finds duplicated ids.

elsif ($func == 6) {
    my %id;
    while (my $x = getutent) {
	if ($id{$x->ut_id}) {
	    show $id{$x->ut_id} if ref $id{$x->ut_id};
	    show $x;
	    $id{$x->ut_id} = 1;
	}
	else {
	    $id{$x->ut_id} = $x;
	}
    }
}

else {
    die "Unknown function $func";
}
